;**************************************************************************
;*            				 Dell'Era Maurizio        	  *
;**************************************************************************

#include 	<xc.inc>
processor     	12f675        ; list directive to define processor	
	
  CONFIG  FOSC = INTRCIO        ; Oscillator Selection bits (INTOSC oscillator: I/O function on GP4/OSC2/CLKOUT pin, I/O function on GP5/OSC1/CLKIN)
  CONFIG  WDTE = OFF            ; Watchdog Timer Enable bit (WDT disabled)
  CONFIG  PWRTE = OFF           ; Power-Up Timer Enable bit (PWRT disabled)
  CONFIG  MCLRE = OFF           ; GP3/MCLR pin function select (GP3/MCLR pin function is digital I/O, MCLR internally tied to VDD)
  CONFIG  BOREN = OFF           ; Brown-out Detect Enable bit (BOD disabled)
  CONFIG  CP = OFF              ; Code Protection bit (Program Memory code protection is disabled)
  CONFIG  CPD = OFF             ; Data Code Protection bit (Data memory code protection is disabled)

	
;==========================================================================
;       Variable Definition
;==========================================================================
STATO	    EQU		20h
ACCUMUL	    EQU		21h
CONT_1	    EQU		22h
CONT_2	    EQU		23h
CONTATORE   EQU		24h
HOLD_STATUS EQU		25h

#define	    ZERO_CROSSING   GPIO,2
#define	    ABILITAZIONE    GPIO,0
#define	    USCITA	    GPIO,1


PSECT   code




;********************************************************
;*			PARTE AGGIUNTA			*
;********************************************************


	PSECT RES_VECT, class=code, delta=2
RES_VECT: 	GOTO INIZIA

	PSECT INT_VECT, class=code, delta=2
INT_VECT:	GOTO ISR
   

;********************************************************
;*		FINE PARTE AGGIUNTA			*
;********************************************************




INIZIA:	    CALL    RESETTA		; Vai alla routine di inizializzazione
	    GOTO    MAIN		; Vai al programma principale

;		*******************************************
;		**	Routine di interrupt		 **
;		*******************************************

ISR:	    MOVWF   ACCUMUL		; Memorizza il valore di Word
	    SWAPF   STATUS,0		; Memorizza il valore di STATUS
	    MOVWF   HOLD_STATUS		;  nella variabile HOLD_STATUS
	    BCF	    INTCON,7		; Disabilita le tutte le interruzioni
	    BTFSC   INTCON,2		; Controlla che sia il Timer 0 a
	    GOTO    TIMER_0		;  richiedere l'interruzione di programma
	    BTFSC   INTCON,1		; Controlla che sia una variazione della 
	    GOTO    POR_0		;  porta GP2
	    GOTO    INT_1
									
TIMER_0:    BCF	    INTCON,2		; Resetta la variabile  Timer 0
	    CLRF    TMR0
	    BTFSS   STATO,0
	    GOTO    TIM_1
	    INCF    CONT_2,1
	    GOTO    TIM_3
TIM_1:	    CLRF    CONT_2
		
TIM_3:	    BSF	    STATUS,5		; Cambia banco di memoria
	    MOVLW   11000000B		; Set PIC options (See datasheet).
	    MOVWF   OPTION_REG
	    BCF	    STATUS,5		; Cambia banco di memoria
	    GOTO    INT_1		; Termina riconfigurando i registri 
					;  principali (Word, Status)
POR_0:	    BCF	    INTCON,1
	    clrf    TMR0
	    BSF	    STATO,0
	    BCF	    USCITA
	    BCF	    STATO,7
	    GOTO    INT_1		; Termina riconfigurando i registri 
					;  principali (Word, Status)

INT_1:	    SWAPF   HOLD_STATUS,0
	    MOVWF   STATUS
	    SWAPF   ACCUMUL,1
	    SWAPF   ACCUMUL,0		; Ripristina il valore di Word
	    RETFIE			; Ritorna al programma principale

;******************************************************
;*  Inizio programma pricipale che viene ripetuto     *
;*   			all'infinito		      *
;******************************************************

MAIN:	    CLRF    TMR0

MAIN_2:	    MOVF    CONTATORE,0
	    XORLW   10
	    BTFSS   STATUS,2
	    GOTO    MAIN_3
	    MOVLW   1
	    MOVWF   CONTATORE

MAIN_3:	    MOVF    CONTATORE,0
	    XORLW   0
	    BTFSS   STATUS,2
	    GOTO    MAIN_9
	    MOVLW   1
	    MOVWF   CONTATORE

MAIN_9:	    BTFSS   ABILITAZIONE
	    GOTO    MAIN_4
	    BSF	    USCITA
	    GOTO    MAIN_2

MAIN_4:	    MOVF    CONT_2,0
	    XORLW   6
	    BTFSS   STATUS,2
	    GOTO    MAIN_5
	    BSF	    USCITA
	    BCF	    STATO,0
	    bsf	    STATO,7

MAIN_5:	    GOTO    MAIN_2


;	    *******************************************
;           **	Questa routine serve per impostare   **
;	    **	i registri di configurazione  	     **
;           *******************************************

RESETTA:    BSF	    STATUS,5		; Cambia il banco di memoria
					; Disabilita pull-ups
					; INT on falling edge
					; TMR0 to CLKINT
					; TMR0 Incr low2high trans.
					; Prescaler  assegnato al Timer0
					; Prescaler impostato 1:2
	    MOVLW   01000000B		; Set PIC options (See datasheet).
	    MOVWF   OPTION_REG		; Write the OPTION register.
	    
	    MOVLW   00110101B
	    MOVWF   TRISIO

	    MOVLW   00000100B		; Abilita l'interruzione solo per il
	    MOVWF   IOC			;  cambiamento della porta GPIO2

	    MOVLW   10110000B
	    MOVWF   INTCON

	    MOVLW   00110000B
	    MOVWF   WPU

	    MOVLW   10000000B		; Imposta l'oscillatore interno
	    MOVWF   OSCCAL		;  alla frequenza intermedia (4 Mhz)

	    MOVLW   00000000B		; Imposta i piedini come entrate e uscite
	    MOVWF   ANSEL		;  digitali (disabilita il convertitore)

	    BCF	    STATUS,5		; Cambia il banco di memoria
	    
	    MOVLW   00000000B
	    MOVWF   GPIO

	    MOVLW   00000111B
	    MOVWF   CMCON

	    CALL    CLEAR_BIT		; Esegui la pulitura delle variabili

	    RETURN

;******************************************************
;*  Routine che resetta le variabili interessate      *
;*               dal programma	       		      *
;******************************************************

CLEAR_BIT:   MOVLW   0X20		; Parte dall'indirizzo H'20'che
	    MOVWF   FSR			; viene caricato in FSR
NEXT_1:	    CLRF    INDF		; Cancella in contenuto della variabile
	    INCF    FSR,1		;  puntatada FSR
	    BTFSS   FSR,6		; Fino a che FSR non arriva a H'50'
	    GOTO    NEXT_1
	    RETURN  
	    
    end 